from __future__ import division, print_function

import numpy as np

has_simpleaudio=True
try:
    import simpleaudio
except:
    print("Cannot find simpleaudio package. Not installed?")
    print('Try:\n(sudo) pip install --upgrade simpleaudio')
    has_simpleaudio=False



def soundof(notes, duration, volume=1):

    sample_rate = 44100

    # calculate note frequencies
    intensities = []
    for n in notes:
        # freq = frequency(n)
        n21 = n.note21
        if hasattr(n21, 'pitch'):
            freq = n21.pitch.frequency
        else:
            freq = n21.frequency
    
        # get timesteps for each sample, duration is note duration in seconds
        t = np.linspace(0, duration, duration * sample_rate, False)

        # generate sine wave notes
        intensity = np.sin((freq*2*np.pi) * t )
        intensities.append(intensity)
    
    for i in range(1, len(intensities)):
        intensity = intensity + intensities[i]

    if not 0<volume<=1: volume=1

    audio = intensity/(len(intensities)/float(volume))

    # concatenate notes
    # audio = np.hstack((intensity1, intensity2))
    # normalize to 16-bit range
    audio *= 32767 / np.max(np.abs(audio))
    # convert to 16-bit data
    audio = audio.astype(np.int16)

    # start playback
    play_obj = simpleaudio.play_buffer(audio, 1, 2, sample_rate)

    # wait for playback to finish before exiting
    play_obj.wait_done()

    
def pitch(freq):
    from math import log2, pow

    A4 = 440
    C0 = A4*pow(2, -4.75)
    name = ["C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"]
    h = round(12*log2(freq/C0))
    octave = h // 12
    n = h % 12
    return name[n] + str(octave)
    

####################################################### 
from music21.midi.realtime import StreamPlayer
from music21.stream import Stream

def playSound(n, speedfactor):
    if has_simpleaudio:
        soundof([n], n.duration/speedfactor)
    else:
        try:
            s = Stream() 
            if n.isChord: n = n.chord21
            else: s.append(n.note21)
            sp = StreamPlayer(s)
            sp.play()            
            # if n.isChord: 
            #     s.append(n)
            # else: 
            #     nn = Note(n.nameWithOctave)
            #     s.append(nn)
            # sp = StreamPlayer(s)
            # sp.play()        
        except:
            print('Unable to play sounds, add -z option')
        return
    


__author__  = "Marco Musy"
__email__   = "marco.musy@gmail.com"
__version__ = "2.0.5" # defined in setup.py and README.md
__status__  = "dev"
__website__ = "https://github.com/marcomusy/pianoplayer"
__license__ = "MIT" 



#
# -*- coding: utf-8 -*-
"""
Created on Thu Nov 26 19:22:20 2015

@author: marco musy
"""
from __future__ import division, print_function
from pianoplayer.utils import keypos

#####################################################
class INote:
    def __init__(self):
        self.note21   = None
        self.name     = None
        self.isChord  = False
        self.isBlack  = False
        self.octave   = 0
        self.x        = 0.0
        self.time     = 0.0
        self.duration = 0.0
        self.fingering= 0 
        self.measure  = 0
        self.chordnr  = 0
    

#####################################################
def reader(sf, beam=0):
    
    try:
        strm = sf.parts[beam].flat
    except AttributeError: 
        strm = sf.flat
    except IndexError:
        print('Beam nr.', beam, 'not found. Skip.')
        return []
    print('Reading beam', beam, 'with', len(strm), 'objects in stream.')

    noteseq = []

    for n in strm:

        if n.duration.quarterLength==0 : continue
        
        if n.tie and (n.tie.type=='continue' or n.tie.type=='stop'): continue

        if n.isNote:
            if len(noteseq) and n.offset == noteseq[-1].time:
                # print "doppia nota", n.name
                continue
            an        = INote()
            an.note21 = n
            an.isChord= False
            an.name   = n.name
            an.octave = n.octave
            an.measure= n.measureNumber
            an.x      = keypos(n)
            an.time   = n.offset
            an.duration = n.duration.quarterLength
            an.isBlack= False
            if hasattr(n, 'pitch'):
                pc = n.pitch.pitchClass
            else:
                pc = n.pitchClass
                an.isBlack = False
            if pc in [1, 3, 6, 8, 10]: an.isBlack = True
            if n.lyrics: an.fingering = n.lyric
            noteseq.append(an)

        if n.isChord:
            if n.tie and (n.tie.type=='continue' or n.tie.type=='stop'): continue
            sfasam = 0.005 # sfasa leggermente le note dell'accordo
            for j, cn in enumerate(n.pitches):
                an = INote()
                an.isChord = True
                an.chord21 = n
                an.note21  = cn
                an.name    = cn.name
                an.chordnr = j
                an.octave  = cn.octave
                an.measure = n.measureNumber
                an.x       = keypos(cn)
                an.time    = n.offset                 +sfasam*j
                an.duration= n.duration.quarterLength -sfasam*(len(n.pitches)-1)
                if hasattr(cn, 'pitch'):
                    pc = cn.pitch.pitchClass
                else:
                    pc = cn.pitchClass
                if pc in [1, 3, 6, 8, 10]: an.isBlack = True
                else: an.isBlack = False
                noteseq.append(an)

    if len(noteseq)<2: 
        print("Beam is empty. Exit.")
        quit()    
    
    return noteseq




#
# -*- coding: utf-8 -*-
"""
Created on Thu Nov 26 19:22:20 2015

@author: marco musy
"""


def nameof(n):
    a = n.name+str(n.octave)
    if "-" in a:
        b = a.replace("C-","B")
        b = b.replace("D-","C#")
        b = b.replace("E-","D#")
        b = b.replace("F-","E")
        b = b.replace("G-","F#")
        b = b.replace("A-","G#")
        b = b.replace("B-","A#")
        return b 
    elif "E#" in a:
        b = a.replace("E#","F")
        return b         
    elif "B#" in a:
        b = a.replace("B#","C")
        return b         
    else:
        return a

def fpress(f, color):
    f.rotate(-20, axis=(1,0,0), axis_point=f.pos())
    f.addPos([0,0,-1])
    f.color(color)#.alpha(1)

def frelease(f):
    f.addPos([0,0,1])
    f.rotate(20, axis=(1,0,0), axis_point=f.pos())
    f.color((.7,0.3,0.3))#.alpha(.6)

def kpress(f, color):
    f.rotate(4, axis=(1,0,0), axis_point=f.pos())
    f.addPos([0,0,-.4])
    f.color(color)

def krelease(f):
    f.addPos([0,0,.4])
    p = f.pos()
    f.rotate(-4, axis=(1,0,0), axis_point=p)
    if p[2]>.5: f.color('k')
    else: f.color('w')


def keypos(n): #position of notes on keyboard
    step = 0.0
    keybsize = 16.5 #cm
    k = keybsize/7.0
    if   n.name == 'C'  : step = k *0.5
    elif n.name == 'D'  : step = k *1.5
    elif n.name == 'E'  : step = k *2.5
    elif n.name == 'F'  : step = k *3.5
    elif n.name == 'G'  : step = k *4.5
    elif n.name == 'A'  : step = k *5.5
    elif n.name == 'B'  : step = k *6.5
    elif n.name == 'B#' : step = k *0.5
    elif n.name == 'C#' : step = k
    elif n.name == 'D#' : step = k *2.
    elif n.name == 'E#' : step = k *3.5
    elif n.name == 'F#' : step = k *4.
    elif n.name == 'G#' : step = k *5.
    elif n.name == 'A#' : step = k *6.
    elif n.name == 'C-' : step = k *6.5
    elif n.name == 'D-' : step = k 
    elif n.name == 'E-' : step = k *2.
    elif n.name == 'F-' : step = k *2.5
    elif n.name == 'G-' : step = k *4.
    elif n.name == 'A-' : step = k *5.
    elif n.name == 'B-' : step = k *6.
    else: 
        print("ERROR note not found", n.name)
    return keybsize * n.octave + step


def handSizeFactor(s): 
    f=0.82       
    if    s=='XXS': f = 0.33
    elif  s=='XS' : f = 0.46
    elif  s=='S'  : f = 0.64
    elif  s=='M'  : f = 0.82
    elif  s=='L'  : f = 1.0
    elif  s=='XL' : f = 1.1
    elif  s=='XXL': f = 1.2
    return f



















from setuptools import setup

setup(
    name='pianoplayer',
    version='2.0.5', #change also in __init__.py and README.md
    packages=['pianoplayer'],
    scripts=['bin/pianoplayer'],
    install_requires=['music21','vtkplotter'], # vtk and numpy are needed but better install them manually
    description='Automatic piano fingering generator. Finds and shows in 3D the best fingering combination to play a score.',
    long_description="""Automatic piano fingering generator. 
    Finds and shows in a 3D visualization the best fingering combination to play a score.

    Check out https://github.com/marcomusy/pianoplayer for documentation.""",
    author='Marco Musy',
    author_email='marco.musy@gmail.com',
    license='MIT',
    url='https://github.com/marcomusy/pianoplayer',
    keywords='piano fingering',
    classifiers=['Intended Audience :: Science/Research',
                'Intended Audience :: Education',
                'Programming Language :: Python',
                'License :: OSI Approved :: MIT License',
                'Topic :: Scientific/Engineering :: Visualization',
                'Topic :: Scientific/Engineering :: Information Analysis'],
    include_package_data=True
)

##############################################################
# cd ~/Projects/pianoplayer/
# pip install .

# # check version number here and in __init__

# ana3
# git status
# python setup.py sdist bdist_wheel

# git commit -a -m 'comment'
# git push
# twine upload dist/pianoplayer-?.?.?.tar.gz -r pypi

# # check status at  https://pypi.org/project/pianoplayer/
# # check status at  https://github.com/marcomusy/pianoplayer